<?php

namespace Drupal\smart_date\Commands;

use Drush\Commands\DrushCommands;

/**
 * A drush command file.
 *
 * @package Drupal\smart_date\Commands
 */
class SmartDateDrushCommands extends DrushCommands {

  /**
   * Drush command to migrate core fields to Smart Date fields.
   *
   * @param string $bundle
   *   Content type or other bundle whose data will be used.
   * @param string $dest
   *   Field data will be copied to.
   * @param string $source_start
   *   Field to copy data from.
   * @param string $source_end
   *   Field to copy data from.
   * @param string $source_all_day
   *   Field to copy data from.
   * @command smart_date:migrate
   * @alises sdm
   * @option clear
   *   Clear any data in the destination field.
   * @option entity
   *   Which entity to use as the destination.
   * @option default_duration
   *   If no end date, assumed duration.
   * @option langcode
   *   Language code to store.
   */
  public function migrate($bundle, $dest, $source_start, $source_end = NULL, $source_all_day = NULL, $options = ['clear' => FALSE, 'entity' => 'node', 'default_duration' => 0, 'langcode' => 'en']) {
    $this->output()->writeln('Starting date migration.');
    // TODO: Sanitize provide input.
    $entity = $options['entity'];
    $dest_table = $entity . '__' . $dest;
    $def_duration = (int) $options['default_duration'];
    $lang = $options['langcode'];

    $connection = \Drupal::service('database');
    if ($options['clear']) {
      $connection->truncate($dest_table)->execute();
    }

    // Get all events.
    $events = \Drupal::entityTypeManager()->getStorage($entity)
      ->loadByProperties(['type' => $bundle]);

    $utc = new \DateTimeZone('UTC');

    foreach ($events as $event) {
      $start_date = current($event->get($source_start)->getValue())['value'];

      // If a field was provided to check for all day, check it.
      if ($source_all_day) {
        $all_day = current($event->get($source_all_day)->getValue())['value'];
      }
      else {
        $all_day = FALSE;
      }

      if (!empty($all_day)) {
        $date = new \DateTime(substr($start_date, 0, -8) . '00:00:00', $utc);
        $date = $date->format('U');

        $start_date = $date;
        $end_date = $date + 86340;
        $duration = 1439;
      }
      else {
        $start_date = new \DateTime($start_date, $utc);
        $start_date = $start_date->format('U');

        // Remove any seconds from the incoming value.
        $start_date -= $start_date % 60;

        $needs_end = TRUE;

        if ($source_end && $source_start != $source_end) {
          $end_date = current($event->get($source_end)->getValue())['value'];
        }
        else {
          // Assume a datetime range, so look for the end_value.
          $end_date = current($event->get($source_start)->getValue());
          if (!empty($end_date['end_value'])) {
            $end_date = $end_date['end_value'];
          }
          else {
            $end_date = NULL;
          }
        }

        if (!empty($end_date)) {
          $end_date = new \DateTime($end_date, $utc);
          $end_date = $end_date->format('U');

          // Remove any seconds from the incoming value.
          $end_date -= $end_date % 60;

          // If valid end date, set duration and turn off calculated end.
          if ($start_date < $end_date) {
            $needs_end = FALSE;
            $duration = round(($end_date - $start_date) / 60);
          }
        }

        if ($needs_end) {
          // If the end date is bogus, use default duration.
          $end_date = $start_date + ($def_duration * 60);
          $duration = $def_duration;
        }

      }
      // Insert the resulting data.
      $result = $connection->insert($dest_table)
        ->fields([
          'bundle' => $bundle,
          'deleted' => 0,
          'entity_id' => $event->id(),
          'revision_id' => $event->getRevisionId(),
          'langcode' => $lang,
          'delta' => 0,
          $dest . '_value' => $start_date,
          $dest . '_end_value' => $end_date,
          $dest . '_duration' => $duration,
          $dest . '_rrule' => NULL,
          $dest . '_rrule_index' => NULL,
          $dest . '_timezone' => '',
        ])
        ->execute();
    }

    drupal_flush_all_caches();
  }

}
